package cn.gygxzc.tina.config.shiro.cache

import cn.gygxzc.tina.cache.redis.RedisKeyGenerator
import cn.gygxzc.tina.config.properties.ShiroSessionProperties
import org.apache.shiro.session.Session
import org.apache.shiro.session.mgt.eis.AbstractSessionDAO
import org.slf4j.LoggerFactory
import org.springframework.data.redis.core.RedisTemplate
import java.io.Serializable
import java.nio.charset.Charset
import java.util.concurrent.TimeUnit


/**
 * Shiro SessionDAO
 * 若继承自 AbstractSessionDAO 则不会将Session缓存(shiro-activeSessioCache:xxxx)
 * 继承自 EnterpriseCacheSessionDAO 或 CachingSessionDAO 则自动将session缓存
 * 由于本项目使用Redis作为缓存框架,所以不需要额外缓存开销
 * @author NorthLan
 * @date 2018/4/24
 * @url https://noahlan.com
 */
class ShiroRedisSessionDAO(private val redisTemplate: RedisTemplate<String, Any>,
                           private val properties: ShiroSessionProperties,
                           private var charset: Charset = Charsets.UTF_8) : AbstractSessionDAO() {
    companion object {
        private val logger = LoggerFactory.getLogger(ShiroRedisSessionDAO::class.java)
    }

    private fun getKey(originalKey: String): String {
        return "${properties.prefix}${RedisKeyGenerator.SEPARATOR}$originalKey"
    }

    override fun update(session: Session) {
        logger.debug("Update Session: ${session.id}")
        redisTemplate.opsForValue().set(getKey(session.id.toString()), session, properties.expired.seconds, TimeUnit.SECONDS)
    }

    override fun getActiveSessions(): MutableCollection<Session> {
        logger.debug("Get Active Sessions")
        val result = mutableListOf<Session>()
        val keys = redisTemplate.keys(getKey("*"))
        if (keys.isNotEmpty()) {
            keys.forEach {
                val session = redisTemplate.opsForValue().get(it) as? Session
                if (session != null) {
                    result.add(session)
                }
            }
        }
        return result
    }

    override fun doReadSession(sessionId: Serializable): Session? {
        logger.debug("Read Session: $sessionId")
        return redisTemplate.opsForValue().get(getKey(sessionId.toString())) as? Session
    }

    /**
     * 创建 session
     */
    override fun doCreate(session: Session): Serializable {
        val sid = this.generateSessionId(session)
        this.assignSessionId(session, sid)
        logger.debug("Create Session: $sid")
        redisTemplate.opsForValue().set(getKey(sid.toString()), session, properties.expired.seconds, TimeUnit.SECONDS)
        return sid
    }

    override fun delete(session: Session) {
        logger.debug("Delete Session: ${session.id}")
        redisTemplate.delete(getKey(session.id.toString()))
    }
}